﻿using Common;
using Dapper;
using Newtonsoft.Json.Linq;
using RedisHelp;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Web;
using System.Web.Http;
using TGood.Xian.LogManager;
using ThoughtWorks.QRCode.Codec;
using WxPayAPI.Models;
using wxWebAPI.business;
using wxWebAPI.Common;
using wxWebAPI.Filters;
using wxWebAPI.Models;

namespace WxPayAPI
{
    public class StationController : ApiController
    {

        #region 局站相关
        /// <summary>
        /// 获取周边局站信息
        /// </summary>
        /// <param name="userToken"></param>
        /// <param name="longitude"></param>
        /// <param name="latitude"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetStation(string userToken, double longitude, double latitude)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<dynamic> stations = chargeLogic.GetStation(userToken, longitude, latitude);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(stations));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetStation Error,userToken={userToken},longitude={longitude},latitude={latitude}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 获取局站明细信息
        /// </summary>
        /// <param name="stationCode"></param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetStationDetail(string stationCode, string usertoken = null)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<LCO_Station> stations = chargeLogic.GetStationDetail(stationCode, usertoken);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(stations));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetStationDetail Error,stationCode={stationCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据经纬度和半径获取范围内局站
        /// </summary>
        /// <param name="longitude">经度</param>
        /// <param name="latitude">维度</param>
        /// <param name="radius">查找半径，单位km</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetNearStationSummary(double longitude, double latitude, double radius)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                Dictionary<string, StackExchange.Redis.GeoPosition> stations = chargeLogic.GetNearStationSummary(longitude, latitude, radius);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JArray.FromObject(stations.ToList()));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetNearStationSummary Error,longitude={longitude},latitude={latitude},radius={radius}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据条件查询符合条件的局站
        /// </summary>
        /// <param name="longitude">经度</param>
        /// <param name="latitude">维度</param>
        /// <param name="radius">查找半径，单位km</param>
        /// <param name="stationType">局站类型，值按位从低到高依次代表为公共站、专用站、个人站，全选传0</param>
        /// <param name="chargeGunType">充电枪类型，值按位从低到高依次代表为交流慢充、交流快充、直流快充，全选传0</param>
        /// <param name="isParkingFree">是否免费停车，为null的话不判断该条件</param>
        /// <param name="isWorking">局站是否营业中，依据系统录入的营业时间</param>
        /// <param name="chargeStandardType">支持的充电接口国标类型，国标2011 = 1, 国标2015 = 2, 两者都有 = 3</param>
        /// <param name="auxiliaryPowerType">支持的充电辅源电压类型，辅源12V = 1, 辅源24V = 2, 两者都有 = 3</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetNearStation(double longitude, double latitude, double radius
            , int stationType, int chargeGunType, bool? isParkingFree, bool? isWorking, string companyID = null, string appname = null
            , string usertoken = null, EnumChargeStandardType chargeStandardType = EnumChargeStandardType.两者都有
            , EnumAuxiliaryPowerType auxiliaryPowerType = EnumAuxiliaryPowerType.两者都有
            , EnumStationOrderBy orderby = EnumStationOrderBy.距离最近)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<LCO_Station> stations = chargeLogic.GetNearStation(appname, usertoken, longitude, latitude, radius
                    , stationType, chargeGunType, isParkingFree, isWorking, companyID, chargeStandardType, auxiliaryPowerType, orderby);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(stations));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                var errorMsg = new StringBuilder();
                errorMsg.Append($"GetNearStation Error,longitude={longitude},latitude={latitude},radius={radius},");
                errorMsg.Append($"stationType={stationType},chargeGunType={chargeGunType},isParkingFree={isParkingFree},");
                errorMsg.Append($"isWorking={isWorking},chargeStandardType={chargeStandardType},auxiliaryPowerType={auxiliaryPowerType}");
                LogEvent.WebAPILoger.Error(errorMsg.ToString(), ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 根据经纬度和关键字获取范围内局站(默认半径100公里)
        /// </summary>
        /// <param name="longitude">经度</param>
        /// <param name="latitude">维度</param>
        /// <param name="keyword">关键字</param>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetStationByKeyword(double longitude, double latitude, string keyword, string appname = null)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<LCO_Station> stations = chargeLogic.GetStationByKeyword(longitude, latitude, keyword, appname);
                if (stations != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JArray.FromObject(stations.ToList()));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetNearStationSummary Error,longitude={longitude},latitude={latitude},keyword={keyword}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject GetCurrentElectricity(string stationCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                double? electricity = chargeLogic.GetStationCurrentElectricity(stationCode);
                if (electricity != null)
                {
                    result.Add("code", 1);
                    result.Add("data", electricity);
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetCurrentElectricity Error,stationCode={stationCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject GetTodayElectricity(string stationCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                List<Price> electricity = chargeLogic.GetStationTodayElectricity(stationCode);
                if (electricity != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JArray.FromObject(electricity));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetCurrentElectricity Error,stationCode={stationCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject GetTodayServiceFee(string stationCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                var serviceFee = chargeLogic.GetStationTodayServiceFee(stationCode);
                if (serviceFee != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(serviceFee));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetCurrentElectricity Error,stationCode={stationCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        [HttpGet, HttpOptions]
        public JObject GetTodayChargeStrategy(string stationCode)
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                List<StrategyInfo> chargeStrategy = chargeLogic.GetStationTodayChargeStrategy(stationCode);
                if (chargeStrategy != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JArray.FromObject(chargeStrategy));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error($"GetCurrentElectricity Error,stationCode={stationCode}", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 获取其他信息
        /// </summary>
        /// <returns></returns>
        [HttpGet, HttpOptions]
        public JObject GetOtherInfo(string companyID = "")
        {
            JObject result = new JObject();
            try
            {
                var chargeLogic = new ChargeLogic();
                IEnumerable<string> infos = chargeLogic.GetOtherInfo(companyID);
                if (infos != null)
                {
                    result.Add("code", 1);
                    result.Add("data", JProperty.FromObject(infos));
                }
                else
                {
                    result.Add("code", 0);
                    result.Add("data", null);
                }
            }
            catch (Exception ex)
            {
                LogEvent.WebAPILoger.Error("GetOtherInfo Error", ex);
                result.RemoveAll();
                result.Add("code", 0);
#if DEBUG
                result.Add("error", ex.ToString());
#endif
            }
            return result;
        }

        /// <summary>
        /// 场站评分
        /// </summary>
        /// <returns></returns>
        [HttpPost, HttpOptions]
        public JObject rate()
        {

            var result = new JObject();
            var rating = Int32.Parse(HttpContext.Current.Request["rating"]);

            var stationId = HttpContext.Current.Request["stationId"];
            StationLogic stationLogic = new StationLogic();
            stationLogic.updateRate(stationId, rating);
            try
            {
                stationLogic.CulculateStationScore(stationId);
                result.Add("code", 1);
            }
            catch (Exception e)
            {
                LogEvent.WebAPILoger.Error($"{stationId}不存在！", e);
                result.Add("code", 0);
                result.Add("data", "此站点不存在！");
            }

            return result;
        }

        [HttpGet, HttpOptions]

        public JObject GetMonitorData(string stationId, string usertoken = null)
        {
            JObject result = new JObject();

            var monitorData = StationLogic.GetMonitorDataByStation(usertoken, stationId);
            if (monitorData != null)
            {
                result.Add("code", 1);
                result.Add("result", JArray.FromObject(monitorData));
            }
            else
            {
                result.Add("code", 0);
                result.Add("result", "查无数据！");
            }
            return result;
        }

        private double CalculateAverage(List<string> ratings)
        {
            var list = ratings.Select<string, double>(x => double.Parse(x));
            var rating = Math.Round(list.Sum() / (double)list.Count(), 1);
            return rating;
        }

        [HttpGet, HttpOptions]
        public JObject GetDeviceInfo(string deviceCode, string usertoken)
        {
            JObject result = new JObject();
            try
            {
                string redisOpenid = "";
                RedisHelper redis = new RedisHelper((int)EnumUserRedisNum.WeiXin);
                RedisHelper redis5 = new RedisHelper((int)EnumUserRedisNum.Struct);
                var deviceinfo = redis.HashGet<WXDeviceInfo>(RedisKey.DeviceInfo, deviceCode);
                if (deviceinfo != null)
                {
                    redisOpenid = deviceinfo.Openid;
                }
                bool ISVIN = CheckVIN(deviceCode, usertoken);
                StationLogic logic = new StationLogic();
                GunStatusInfo guninfo = logic.GetDeviceDetailsInfo(deviceCode,usertoken);
                var openid = WechatUtility.GetOpenIDByUserToken(usertoken);
                List<string> openidList = new List<string>();
                List<string> arr = new List<string>();
                string phone = string.Empty;
                if (!string.IsNullOrEmpty(openid))
                {
                    phone = redis.HashGetString(RedisKey.UserAndPhone, openid);
                    if (openid.Length == 11)
                    {
                        phone = openid;
                    }
                }


                //为了兼容app通过微信支付领充openid,小马openid与微信openid兼容的问题 app openid为11位，在chargingdevice中找正在充电的设备 将两个opeid 关联
                if (!string.IsNullOrEmpty(phone))
                {
                    using (IDbConnection conn = WechatUtility.CreateDBConnection())
                    {
                        string sql = $"SELECT openid FROM lco_wxcustomer WHERE phone = '{phone}'";
                        openidList = conn.Query<string>(sql, conn).ToList();
                    }
                }
              
                openidList.Add(openid);
                openidList.Distinct();
                switch (guninfo.status_type)
                {
                    case (int)Common.EnumGunStatus.连接:
                        if (ISVIN == true)
                        {
                            result.Add("code", 5);
                            result.Add("GunState", Enum.GetName(typeof(Common.EnumGunStatus), guninfo.status_type));
                        }
                        else
                        {
                            result.Add("code", 1);
                            result.Add("GunState", Enum.GetName(typeof(Common.EnumGunStatus), guninfo.status_type));
                        }
                        break;
                    case (int)Common.EnumGunStatus.充满:
                        result.Add("code", 1);
                        result.Add("GunState", Enum.GetName(typeof(Common.EnumGunStatus), guninfo.status_type));

                        break;
                    case (int)Common.EnumGunStatus.启动中:
                    case (int)Common.EnumGunStatus.正在启动:
                        if (!openidList.Contains(redisOpenid))
                        {
                            result.Add("code", 0);
                            result.Add("GunState", "被占用");
                        }
                        else
                        {
                            result.Add("code", -2);
                            result.Add("GunState", "启动中");
                        }
                        break;
                    case (int)Common.EnumGunStatus.充电中:
                        if (!openidList.Contains(redisOpenid))
                        {
                            result.Add("code", 0);
                            result.Add("GunState", "被占用");
                        }
                        else
                        {
                            result.Add("code", 99);
                            result.Add("GunState", "当前用户有未结束订单");
                        }
                        break;
                    case -99999:
                    case 99999:
                        result.Add("code", 0);
                        result.Add("GunState", "离线");
                        break;

                    default:
                        result.Add("code", 0);
                        result.Add("GunState", Enum.GetName(typeof(Common.EnumGunStatus), guninfo.status_type));
                        break;
                }
                result.Add("data", JProperty.FromObject(guninfo));
            }
            catch (Exception e)
            {
                result.Add("code", 0);
                result.Add("data", e.ToString());
                LogEvent.WebAPILoger.Error(e);
            }
            return result;
        }
        public bool CheckVIN(string deviceCode, string userToken)
        {
            bool vin = false;
            RedisHelp.RedisHelper redis = new RedisHelp.RedisHelper((int)EnumUserRedisNum.WeiXin);
            string stationid = WechatUtility.GetStationId(deviceCode);

            List<string> VinPhone = redis.HashGet<List<string>>(RedisKey.VINInfo, stationid);
            string openid = WechatUtility.GetOpenIDByUserToken(userToken);
            if (!string.IsNullOrEmpty(openid))
            {
                string phone = redis.HashGetString(RedisKey.UserAndPhone, openid) == null ? string.Empty : redis.HashGetString(RedisKey.UserAndPhone, openid);
                if (VinPhone != null && VinPhone.Contains(phone))
                {
                    vin = true;
                }
            }


            return vin;
        }
        #endregion
    }
}
